'''
FilePath: /python-Sigilyph/sigilyph/__init__.py
Descripttion: 
Author: Yixiang Chen
version: 
Date: 2025-05-13 11:01:07
LastEditors: Yixiang Chen
LastEditTime: 2025-09-25 17:22:30
'''

from sigilyph.core.sigilyph_class import Sigilyph

from sigilyph.core.symbols import all_phone_dict

from sigilyph.core.bert_align import AlignBert

